@@ -51,7 +51,7 @@ where
 }
 
 #[cfg(test)]
-mod test {
+mod tests {
     use super::*;
     use crate::types::PyDict;
 
