@@ -456,7 +456,7 @@ mod hashbrown_hashmap_conversion {
 }
 
 #[cfg(test)]
-mod test {
+mod tests {
     use crate::conversion::IntoPy;
     use crate::types::dict::IntoPyDict;
     #[cfg(not(PyPy))]
@@ -724,7 +724,7 @@ mod test {
         assert_eq!(iter.size_hint(), (v.len() - 1, Some(v.len() - 1)));
 
         // Exhust iterator.
-        while let Some(_) = iter.next() {}
+        for _ in &mut iter {}
 
         assert_eq!(iter.size_hint(), (0, Some(0)));
     }
