@@ -207,7 +207,7 @@ where
 }
 
 #[cfg(test)]
-mod test {
+mod tests {
     use crate::types::PyList;
     use crate::Python;
     use crate::{IntoPy, PyObject, PyTryFrom, ToPyObject};
@@ -366,7 +366,7 @@ mod test {
         assert_eq!(iter.size_hint(), (v.len() - 1, Some(v.len() - 1)));
 
         // Exhust iterator.
-        while let Some(_) = iter.next() {}
+        for _ in &mut iter {}
 
         assert_eq!(iter.size_hint(), (0, Some(0)));
     }
