@@ -26,9 +26,9 @@ impl PyTuple {
         unsafe {
             let ptr = ffi::PyTuple_New(len as Py_ssize_t);
             for (i, e) in elements_iter.enumerate() {
-                #[cfg(not(Py_LIMITED_API))]
+                #[cfg(not(any(Py_LIMITED_API, PyPy)))]
                 ffi::PyTuple_SET_ITEM(ptr, i as Py_ssize_t, e.to_object(py).into_ptr());
-                #[cfg(Py_LIMITED_API)]
+                #[cfg(any(Py_LIMITED_API, PyPy))]
                 ffi::PyTuple_SetItem(ptr, i as Py_ssize_t, e.to_object(py).into_ptr());
             }
             py.from_owned_ptr(ptr)
@@ -43,9 +43,9 @@ impl PyTuple {
     /// Gets the length of the tuple.
     pub fn len(&self) -> usize {
         unsafe {
-            #[cfg(not(Py_LIMITED_API))]
+            #[cfg(not(any(Py_LIMITED_API, PyPy)))]
             let size = ffi::PyTuple_GET_SIZE(self.as_ptr());
-            #[cfg(Py_LIMITED_API)]
+            #[cfg(any(Py_LIMITED_API, PyPy))]
             let size = ffi::PyTuple_Size(self.as_ptr());
             // non-negative Py_ssize_t should always fit into Rust uint
             size as usize
@@ -79,9 +79,9 @@ impl PyTuple {
     pub fn get_item(&self, index: usize) -> &PyAny {
         assert!(index < self.len());
         unsafe {
-            #[cfg(not(Py_LIMITED_API))]
+            #[cfg(not(any(Py_LIMITED_API, PyPy)))]
             let item = ffi::PyTuple_GET_ITEM(self.as_ptr(), index as Py_ssize_t);
-            #[cfg(Py_LIMITED_API)]
+            #[cfg(any(Py_LIMITED_API, PyPy))]
             let item = ffi::PyTuple_GetItem(self.as_ptr(), index as Py_ssize_t);
 
             self.py().from_borrowed_ptr(item)
@@ -315,7 +315,7 @@ tuple_conversion!(
 );
 
 #[cfg(test)]
-mod test {
+mod tests {
     use crate::types::{PyAny, PyTuple};
     use crate::{PyTryFrom, Python, ToPyObject};
     use std::collections::HashSet;
